import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class Simulator implements ActionListener {
   private BlockSpringConfiguration bsConfig;
   private float t;  // Simulation time time
   private Timer pase;
   private float delta_t;         // in seconds
   private float samplingPeriod;  // in seconds
   
   public Simulator (BlockSpringConfiguration bsc) {
      t=0;
      bsConfig = bsc;
      delta_t = 0.0001f;        // 0,1 [ms]
      samplingPeriod = 0.08f;  //   80 [ms]
   }
   public void setDelta_t(float delta) {
      delta_t = delta;
   }
   public void setSamplingPeriod (float sp) {
      samplingPeriod = sp;
      if (pase!=null)
         if (pase.isRunning()){
            stop();
            start();
         }   
   }
   public void start() {
      t=0;
      stop();
//      pase = new Timer((int)(samplingPeriod*1000/10), this); I used it 
//               to speed the time up and reach the problem sooner.
      pase = new Timer((int)(samplingPeriod*1000), this);
      pase.start();
   }
   public void stop(){
      if (pase!=null)
         pase.stop();
   }
   public void actionPerformed (ActionEvent e) {
      float nextStop=t+samplingPeriod;
      for (; t<nextStop; t+=delta_t){
         bsConfig.computeNextState(delta_t);
         bsConfig.updateState();
      }
      t-=samplingPeriod;
 //  t   t   t   t   t   t   t
 //  |---+---+---+|--+---+---+-|-+---+---+---+--|+---+--
 //            nextStop     nextStop
 // t-= samplingPeriod we keep t within the order of magnitude of delta_t
 // More:
 //    floating point representation of float uses 1 bit for sign, 8 bits for 
 //    exponent, and 23 bits for significand which is 2^23 ~ 10^7 that is 7 
 //    decimal digit. With delta_t=0,0001, after apprx. 35 second, 
 //    we have 2100,000 second; therefore, 2100,000 + 0,0001 =2100,000 in 7 
 //    decimal digit presition. That explains why we can not keep adding to t
 //    without taking the risk it will reach a value for which delta_t can not 
 //    add anymore.
 //    the solution was to keep t closer to delta_t.
 // Another solution would have been to define t, delta_t and samplingPeriod 
 // as double. It defers the problem but won't solve it. It would be OK 
 // for solution that works for a number of days.
 
   }
}
